# -*- encoding : utf-8 -*-

require 'httparty'

require_relative 'headers'
require_relative 'project'
require_relative 'config'
require_relative 'log_store'
require_relative 'index'
require_relative 'shard'
require_relative 'shipper'
require_relative 'consumer_group'
require_relative 'protobuf'

module AliyunLogRubySdk
  class Client

    include HTTParty

    include Headers
    include Project
    include Config
    include LogStore
    include Index
    include Shard
    include Shipper
    include ConsumerGroup

    include Protobuf

    delegate :endpoint,
             :access_key_id,
             :access_key_secret,
             :project_name,
             :debug,
             to: :'AliyunLogRubySdk.config'

    def initialize
      self.class.base_uri(base_uri)
    end

    def get(path, options = {})
      options[:headers] ||= {}

      headers = generate_headers(path: path,
                                 query: options[:query],
                                 headers: options[:headers])

      response = self.class.get(path, query: options[:query], headers: headers)

      if debug?
        response_string = "GET #{base_uri}#{path}, query: #{options[:query].to_param}, headers: #{headers}"

        response_string << ", response: #{response.parsed_response}" if options[:headers]['Content-Type'] != 'application/x-protobuf'

        puts response_string
      end

      response
    end

    def post(path, options = {})
      options[:headers] ||= {}
      options[:body] ||= {}

      if options[:headers]['Content-Type'] == 'application/x-protobuf'
        body = options[:body]
      else
        body = options[:body].transform_keys do |key|
          key.to_s.camelize(:lower)
        end

        body = body.to_json if body.present?
      end

      headers = generate_headers(verb: 'POST',
                                 path: path,
                                 headers: options[:headers],
                                 body: body)

      response = self.class.post(path, headers: headers, body: body)

      if debug?
        response_string = "POST #{base_uri}#{path}, headers: #{headers}"

        if options[:headers]['Content-Type'] != 'application/x-protobuf'
          response_string << ", body: #{body}, response: #{response.parsed_response}"
        end

        puts response_string
      end

      response
    end

    def put(path, options = {})
      raise NotImplementedError
    end

    def delete(path, options = {})
      options[:body] ||= {}

      body = options[:body].transform_keys do |key|
        key.to_s.camelize(:lower)
      end

      body = body.to_json if body.present?

      headers = generate_headers(verb: 'DELETE',
                                 path: path,
                                 headers: options[:headers],
                                 body: body)

      response = self.class.delete(path, headers: headers, body: body)

      puts "DELETE #{base_uri}#{path}, headers: #{headers}, body: #{body}, response: #{response.parsed_response}" if debug?

      response
    end

    private

    def base_uri
      "http://#{project_name}.#{endpoint}"
    end

    def debug?
      debug
    end

    def log_store_name(options = {})
      options[:log_store_name] || AliyunLogRubySdk.config.log_store_name
    end

  end
end
