# -*- encoding : utf-8 -*-

require 'base64'
require 'openssl'

require_relative 'version'
require_relative 'protobuf'

module AliyunLogRubySdk
  module Headers
    X_LOG_SIGNATURE_METHOD = 'hmac-sha1'.freeze

    def generate_headers(options = {})
      headers = options[:headers] || {}
      host = headers.delete('x-log-host')

      headers = headers.merge('x-log-apiversion' => X_LOG_API_VERSION,
                              'x-log-signaturemethod' => X_LOG_SIGNATURE_METHOD,
                              'Date' => Time.now.httpdate,
                              'Host' => (host || "#{project_name}.#{endpoint}"))

      body = options[:body]

      if body.present?
        headers['Content-Length'] = body.bytesize.to_s
        headers['Content-MD5'] = calculate_md5(body)
      else
        headers['Content-Length'] = '0'
        headers['x-log-bodyrawsize'] = '0'
      end

      headers['Authorization'] = generate_authorization(options.merge(headers: headers))

      headers['x-log-date'] = headers['Date']

      headers
    end

    private

    def generate_authorization(options)
      "LOG #{access_key_id}:#{generate_signature(options)}"
    end

    def generate_signature(options)
      Base64.encode64(OpenSSL::HMAC.digest('sha1', access_key_secret, generate_sign_string(options).encode('utf-8'))).rstrip
    end

    def generate_sign_string(options)
      headers = options[:headers]

      %W[
        #{options[:verb] || 'GET'}
        #{headers['Content-MD5']}
        #{headers['Content-Type']}
        #{headers['Date']}
        #{canonicalized_log_headers(headers)}
        #{canonicalized_log_resource(options[:path], options[:query])}
      ].join("\n")
    end

    def calculate_md5(body)
      Digest::MD5.hexdigest(body).upcase
    end

    def canonicalized_log_headers(headers)
      headers.select { |key, _value| key.match(/x-log-*|x-acs-*/) }.map do |key, value|
        "#{key.downcase}:#{value}"
      end.sort.join("\n")
    end

    def canonicalized_log_resource(path, query = {})
      # 这里有一个很奇葩的地方：如果参数的值中存在 =，to_param 方法会将其转变成 %3D，但是阿里云日志签名时是用 = 做签名，因此这里替换回来
      query.present? ? "#{path}?#{query.to_param.gsub('%3D', '=')}" : path
    end
  end
end
