# -*- encoding : utf-8 -*-

module AliyunLogRubySdk
  module Shard
    def list_shards(options = {})
      get("/logstores/#{log_store_name(options)}/shards", options)
    end

    def get_cursor(options = {})
      options[:query] ||= {}
      options[:query][:type] = 'cursor'

      get("/logstores/#{log_store_name(options)}/shards/#{options[:shard_id]}", options)
    end

    def put_logs(options = {})
      body = generate_log_group(options).encode.to_s

      raise AliyunLogRubySdk::PostBodyTooLarge, 'Post body is larger then 3MB' if body.length > 3 * 1024 * 1024

      options[:headers] = {
        'Content-Type' => 'application/x-protobuf',
	      'x-log-bodyrawsize' => body.length.to_s,
	      'x-log-compresstype' => 'deflate'
      }

      options[:body] = Zlib::Deflate.deflate(body)

      post("/logstores/#{log_store_name(options)}/shards/lb", options)
    end

    def pull_logs(options = {})
      options[:headers] = {
        'Accept' => 'application/x-protobuf',
        'Accept-Encoding' => 'gzip',
        'Content-Type' => 'application/x-protobuf',
      }

      options[:query] ||= {}
      options[:query][:type] = 'log'

      parse(get("/logstores/#{log_store_name(options)}/shards/#{options[:shard_id]}", options))
    end

    private

    def generate_log_group(options)
      log_group = Protobuf::LogGroup.new(logs: [], topic: options[:topic], source: options[:source])

      options[:log_items].each do |log_item|
        log = Protobuf::Log.new(time: log_item.timestamp, contents: [])

        log_item.contents.each do |content|
          log.contents << Protobuf::Log::Content.new(key: content[0], value: content[1])
        end

        log_group.logs << log
      end

      log_group
    end

    def parse(response)
      begin
        log_group_list = Protobuf::LogGroupList.decode(Zlib::Inflate.inflate(response.body))
      rescue
        raise AliyunLogRubySdk::ParseDataError, 'Failed to parse data to LogGroupList'
      end

      group_list = { detail: [] }

      log_group_list.log_group_list.each do |log_group|
        group = { topic: log_group.topic, source: log_group.source, logs: [] }

        log_group.logs.each do |log|
          l = { time: log.time, contents: [] }

          log.contents.each do |content|
            l[:contents] << [content.key, content.value]
          end

          group[:logs] << l
        end

        group_list[:detail] << group
      end

      puts "response: #{group_list.inspect}" if debug?
    end
  end
end
